__version__ = '1.0.165'
__author__ = 'zhengzhengde  mail: drivener@163.com'

"""
更新日志：
20200808 v1.0.0 初始版本

20210303 v1.0.15 general.letterbox
				general.plot_one_box_trace_pose_status
				general.xyxy2xywh
				general.xywh2xyxy
				dm.
20211010 v1.0.153 dm.tools.check_COCO dm.tools.check_YOLO
20211201 v1.0.154 数据增强，根据mask的目标的随机位置、随机缩放和随机旋转，还没写好啊
20211210 v1.0.155 dm.genral.pts2bbox 
20211223 v1.0.156 dm.video2frames
20211223 v1.0.157 dm.misc.list2table dm.misc.flatten_list
20220114 v1.0.158 dm.general.torch_distributed_zero_first
20220215 v1.0.160

"""
